/* Copyright (C) 2013-2016 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#ifndef FLOAT22_H
#define FLOAT22_H

#include "float2.h"

/* Generate common floatXY funcs */
#define FLOATX_DIMENSION__ 2
#define FLOATY_DIMENSION__ 2
#include "floatXY.h"

/* Specific float22 funcs */
static FINLINE float*
f22(float* dst, const float a, const float b, const float c, const float d)
{
  ASSERT(dst);
  dst[0] = a; dst[1] = b; dst[2] = c; dst[3] = d;
  return dst;
}

static FINLINE float
f22_det(const float* mat)
{
  return mat[0] * mat[3] - mat[2] * mat[1];
}

static FINLINE float
f22_inverse(float* dst, const float* mat)
{
  float det, rcp_det, mat0;
  ASSERT(dst && mat);
  det = f22_det(mat);
  rcp_det = 1.f / det;
  mat0 = mat[0];
  dst[0] =  mat[3] * rcp_det;
  dst[1] = -mat[1] * rcp_det;
  dst[2] = -mat[2] * rcp_det;
  dst[3] =  mat0   * rcp_det;
  return det;
}

static FINLINE float
f22_invtrans(float* dst, const float* mat)
{
  float det, dst1, dst2;
  ASSERT(dst && mat);
  det = f22_inverse(dst, mat);
  dst1 = dst[1];
  dst2 = dst[2];
  dst[1] = dst2;
  dst[2] = dst1;
  return det;
}

#endif /* FLOAT22_H */

