/* Copyright (C) 2013-2016 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#ifndef FLOAT44_H
#define FLOAT44_H

#include "float33.h"
#include "float4.h"

/* Generate common floatXY funcs */
#define FLOATX_DIMENSION__ 4
#define FLOATY_DIMENSION__ 4
#include "floatXY.h"

/* Specific float44 funcs*/
static FINLINE float*
f44
  (float* dst,
   const float a, const float b, const float c, const float d,
   const float e, const float f, const float g, const float h,
   const float i, const float j, const float k, const float l,
   const float m, const float n, const float o, const float p)
{
  ASSERT(dst);
  dst[0] = a;  dst[1] = b;  dst[2] = c;  dst[3] = d;
  dst[4] = e;  dst[5] = f;  dst[6] = g;  dst[7] = h;
  dst[8] = i;  dst[9] = j;  dst[10] = k; dst[11] = l;
  dst[12] = m; dst[13] = n; dst[14] = o; dst[15] = p;
  return dst;
}

static FINLINE float
f44_det(const float* mat)
{
  float f33[9];
  float row3[4], tmp[4];
  ASSERT( mat );

  #define C3( Dst, Mat, ICol ) \
    (Dst)[0] = Mat[ICol*4], (Dst)[1] = Mat[ICol*4+1], (Dst)[2] = Mat[ICol*4+2]
  C3(f33 + 0, mat, 1); C3(f33 + 3, mat, 2);  C3(f33 + 6, mat, 3);
  tmp[0] = -f33_det(f33);
  C3(f33 + 0, mat, 0); C3(f33 + 3, mat, 2);  C3(f33 + 6, mat, 3);
  tmp[1] =  f33_det(f33);
  C3(f33 + 0, mat, 0); C3(f33 + 3, mat, 1);  C3(f33 + 6, mat, 3);
  tmp[2] = -f33_det(f33);
  C3(f33 + 0, mat, 0); C3(f33 + 3, mat, 1);  C3(f33 + 6, mat, 2);
  #undef C3
  tmp[3] =  f33_det(f33);
  f44_row(row3, mat, 3);
  return f4_dot(tmp,  row3);
}

BEGIN_DECLS

RSYS_API float
f44_inverse(float* dst, const float* mat);

END_DECLS

static FINLINE float
f44_invtrans(float* dst, const float* mat)
{
  const float det = f44_inverse(dst, mat);
  f44_transpose(dst, dst);
  return det;
}

#endif /* FLOAT44_H */

