/* Copyright (C) 2013-2016 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#include "cstr.h"
#include "math.h"

int
main(int argc, char** argv)
{
  char buf[128];
  double dlist[4];
  float flist[4];
  size_t len;
  double d;
  float f;
  long l;
  int i;
  unsigned u;
  unsigned long ul;
  (void)argc, (void)argv;

  CHECK(cstr_to_double(NULL, &d), RES_BAD_ARG);
  CHECK(cstr_to_double("a", &d), RES_BAD_ARG);
  CHECK(cstr_to_double(STR(PI), &d), RES_OK);
  CHECK(d, PI);
  CHECK(cstr_to_double("  1.e-3", &d), RES_OK);
  CHECK(d, 1.e-3);
  CHECK(cstr_to_double("+1.E+3 ", &d), RES_OK);
  CHECK(d, 1.e3);
  CHECK(cstr_to_double("INF", &d), RES_OK);
  CHECK(d, INF);
  CHECK(cstr_to_double("INFINITY", &d), RES_OK);
  CHECK(d, INF);
  CHECK(cstr_to_double("", &d), RES_BAD_ARG);

  CHECK(cstr_to_float(NULL, &f), RES_BAD_ARG);
  CHECK(cstr_to_float("a", &f), RES_BAD_ARG);
  CHECK(cstr_to_float(STR(PI), &f), RES_OK);
  CHECK(f, (float)PI);
  CHECK(cstr_to_float("1.e-1", &f), RES_OK);
  CHECK(f, 1.e-1f);
  CHECK(cstr_to_float("+1.E+3", &f), RES_OK);
  CHECK(f, 1.e3f);
  CHECK(cstr_to_float("INF", &f), RES_OK);
  CHECK(f, (float)INF);
  CHECK(cstr_to_float("INFINITY", &f), RES_OK);
  CHECK(f, (float)INF);
  CHECK(cstr_to_float(STR(DBL_MAX), &f), RES_BAD_ARG);
  CHECK(cstr_to_float(STR(DBL_MIN), &f), RES_BAD_ARG);
  CHECK(cstr_to_float("0.0", &f), RES_OK);
  CHECK(f, 0.f);
  CHECK(cstr_to_float("-0.0", &f), RES_OK);
  CHECK(f, 0.f);

  CHECK(cstr_to_long(NULL, &l), RES_BAD_ARG);
  CHECK(cstr_to_long("a", &l), RES_BAD_ARG);
  CHECK(cstr_to_long("1.e-3", &l), RES_BAD_ARG);
  CHECK(cstr_to_long("1", &l), RES_OK);
  CHECK(l, 1);
  CHECK(cstr_to_long("-1", &l), RES_OK);
  CHECK(l, -1);
  CHECK(cstr_to_long("+1234567890", &l), RES_OK);
  CHECK(l, 1234567890);
  CHECK(cstr_to_long("     \t+1234567890  \t  ", &l), RES_OK);
  CHECK(l, 1234567890);
  CHECK(cstr_to_long("     \t+1234567890  \t  a", &l), RES_BAD_ARG);

  CHECK(cstr_to_int(NULL, &i), RES_BAD_ARG);
  CHECK(cstr_to_int("a", &i), RES_BAD_ARG);
  CHECK(cstr_to_int("1.e-1", &i), RES_BAD_ARG);
  CHECK(cstr_to_int("1", &i), RES_OK);
  CHECK(i, 1);
  CHECK(cstr_to_int("-2", &i), RES_OK);
  CHECK(i, -2);
  CHECK(cstr_to_int("\t-2  ", &i), RES_OK);
  CHECK(i, -2);
  sprintf(buf, "%d", INT_MAX);
  CHECK(cstr_to_int(buf, &i), RES_OK);
  CHECK(i, INT_MAX);
  sprintf(buf, "%d", INT_MIN);
  CHECK(cstr_to_int(buf, &i), RES_OK);
  CHECK(i, INT_MIN);
  sprintf(buf, "%lld", (long long)INT_MAX+1);
  CHECK(cstr_to_int(buf, &i), RES_BAD_ARG);
  sprintf(buf, "%lld", (long long)INT_MIN-1);
  CHECK(cstr_to_int(buf, &i), RES_BAD_ARG);

  CHECK(cstr_to_uint(NULL, &u), RES_BAD_ARG);
  CHECK(cstr_to_uint("a", &u), RES_BAD_ARG);
  CHECK(cstr_to_uint("-1", &u), RES_OK);
  CHECK(u, UINT_MAX);
  CHECK(cstr_to_uint("0.", &u), RES_BAD_ARG);
  CHECK(cstr_to_uint("0", &u), RES_OK);
  CHECK(u, 0);
  CHECK(cstr_to_uint("+2", &u), RES_OK);
  CHECK(u, 2);
  CHECK(cstr_to_uint(" \t+123 \t ", &u), RES_OK);
  CHECK(u, 123);
  sprintf(buf, "%u", UINT_MAX);
  CHECK(cstr_to_uint(buf, &u), RES_OK);
  CHECK(u, UINT_MAX);
  sprintf(buf, "%llu", (unsigned long long)UINT_MAX+1);
  CHECK(cstr_to_uint(buf, &u), RES_BAD_ARG);

  CHECK(cstr_to_ulong(NULL, &ul), RES_BAD_ARG);
  CHECK(cstr_to_ulong("a", &ul), RES_BAD_ARG);
  CHECK(cstr_to_ulong("-1", &ul), RES_OK);
  CHECK(ul, ULONG_MAX);
  CHECK(cstr_to_ulong("0.", &ul), RES_BAD_ARG);
  CHECK(cstr_to_ulong("0", &ul), RES_OK);
  CHECK(ul, 0);
  CHECK(cstr_to_ulong("+2", &ul), RES_OK);
  CHECK(ul, 2);
  CHECK(cstr_to_ulong(" \t+123 \t ", &ul), RES_OK);
  CHECK(ul, 123);
  sprintf(buf, "%lu", ULONG_MAX);
  CHECK(cstr_to_ulong(buf, &ul), RES_OK);
  CHECK(ul, ULONG_MAX);
  sprintf(buf, "%lu%c", ULONG_MAX, '0');
  CHECK(cstr_to_ulong(buf, &ul), RES_BAD_ARG);

  CHECK(cstr_to_list_double(NULL, dlist, NULL, 3), RES_BAD_ARG);
  CHECK(cstr_to_list_double("a", dlist, NULL, 3), RES_BAD_ARG);
  CHECK(cstr_to_list_double("1.e-3:2.0:"STR(PI), dlist, NULL, 3), RES_OK);
  CHECK(dlist[0], 1.e-3);
  CHECK(dlist[1], 2.0);
  CHECK(dlist[2], PI);
  CHECK(cstr_to_list_double("1.e-3:2.0:", dlist, &len, 3), RES_OK);
  CHECK(len, 2);
  CHECK(dlist[0], 1.e-3);
  CHECK(dlist[1], 2.0);
  CHECK(cstr_to_list_double("-1.0:0.5:1.2:4.3", dlist, &len, 2), RES_BAD_ARG);
  CHECK(cstr_to_list_double("-1.0:0.5:1.2:4.3a", NULL, &len, 0), RES_BAD_ARG);
  CHECK(cstr_to_list_double("-1.0:0.5:1.2:4.3", NULL, &len, 0), RES_OK);
  CHECK(len, 4);
  CHECK(cstr_to_list_double("-1.0:0.5:1.2:4.3", dlist, NULL, len), RES_OK);
  CHECK(dlist[0], -1.0);
  CHECK(dlist[1], 0.5);
  CHECK(dlist[2], 1.2);
  CHECK(dlist[3], 4.3);
  CHECK(cstr_to_list_double("  \t -1.0:0.5:1.2:INF  \t", dlist, NULL, 4), RES_OK);
  CHECK(dlist[0], -1.0);
  CHECK(dlist[1], 0.5);
  CHECK(dlist[2], 1.2);
  CHECK(dlist[3], INF);
  CHECK(cstr_to_list_double("  \t -1.0:0.5:1.2:4.3  \ta", dlist, NULL, 4), RES_BAD_ARG);
  dlist[1] = dlist[2] = dlist[3] = -1.0;
  CHECK(cstr_to_list_double("1.0", dlist, NULL, 1), RES_OK);
  CHECK(dlist[0], 1.0);
  CHECK(dlist[1], -1.0);
  CHECK(dlist[2], -1.0);
  CHECK(dlist[3], -1.0);

  CHECK(cstr_to_list_float(NULL, flist, NULL, 3), RES_BAD_ARG);
  CHECK(cstr_to_list_float("a", flist, NULL, 3), RES_BAD_ARG);
  CHECK(cstr_to_list_float("1.e-3:2.0:"STR(PI), flist, NULL, 3), RES_OK);
  CHECK(flist[0], 1.e-3f);
  CHECK(flist[1], 2.0f);
  CHECK(flist[2], (float)PI);
  CHECK(cstr_to_list_float("1.e-3:2.0:", flist, &len, 3), RES_OK);
  CHECK(len, 2);
  CHECK(flist[0], 1.e-3f);
  CHECK(flist[1], 2.0f);
  CHECK(cstr_to_list_float("-1.0:0.5:1.2:4.3", flist, &len, 2), RES_BAD_ARG);
  CHECK(cstr_to_list_float("-1.0:0.5:1.2:4.3a", NULL, &len, 0), RES_BAD_ARG);
  CHECK(cstr_to_list_float("-1.0:0.5:1.2:4.3", NULL, &len, 0), RES_OK);
  CHECK(len, 4);
  CHECK(cstr_to_list_float("-1.0:0.5:1.2:4.3", flist, NULL, len), RES_OK);
  CHECK(flist[0], -1.0f);
  CHECK(flist[1], 0.5f);
  CHECK(flist[2], 1.2f);
  CHECK(flist[3], 4.3f);
  CHECK(cstr_to_list_float("  \t -1.0:0.5:1.2:INF  \t", flist, NULL, 4), RES_OK);
  CHECK(flist[0], -1.0f);
  CHECK(flist[1], 0.5f);
  CHECK(flist[2], 1.2f);
  CHECK(flist[3], (float)INF);

  return 0;
}
