/* Copyright (C) 2013-2016 Vincent Forest (vaplv@free.fr)
 *
 * The RSys library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The RSys library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with the RSys library. If not, see <http://www.gnu.org/licenses/>. */

#include "math.h"

#ifdef COMPILER_CL
  #pragma warning(disable:4127) /* Constant conditional expression */
  #pragma warning(disable:4723) /* Division by zero */
#endif

int
main(int argc, char** argv)
{
  float f, g;
  int i;
  (void)argc, (void)argv;

  f = -3.14159f; CHECK(absf(f),-f);
  f =  3.14159f; CHECK(absf(f), f);
  f = -3.14159f; g = 2.71828f;
  CHECK(MMAX(f, g), g);
  CHECK(MMIN(f, g), f);
  CHECK(MMAX(-10, 0), 0);
  CHECK(MMAX(10, 0), 10);
  CHECK(MMIN(-10, 0), -10);
  CHECK(MMIN(10, 0), 0);

  CHECK(CLAMP(0.1f, 0.f, 1.f), 0.1f);
  CHECK(CLAMP(-0.1f, 0.f, 1.f), 0.f);
  CHECK(CLAMP(1.2f, 0.f, 1.f), 1.f);
  CHECK(CLAMP(1.f, 0.f, 1.f), 1.f);
  CHECK(CLAMP(0, -127, 127), 0);
  CHECK(CLAMP(255, -127, 127), 127);
  CHECK(CLAMP(-255, -127, 127), -127);

  CHECK(eq_epsf((float)PI, 3.14159265358979323846f, 1.e-6f), 1);
  CHECK(eq_epsf((float)RCP_PI, 1.f / (float)PI, 1.e-6f), 1);
  CHECK(eq_eps(PI, 3.14159265358979323846, 1.e-8), 1);
  CHECK(eq_eps(RCP_PI, 1.0/PI, 1.e-8), 1);

  CHECK(1.f/0.f, (float)INF);
  CHECK(-1.f/0.f, (float)-INF);
  NCHECK(1.0/0.0, -INF);
  CHECK(1.0/0.0, INF);
  NCHECK(-1.0/0.0, INF);
  CHECK(-1.0/0.0, -INF);
  NCHECK(IS_INF(PI/0.0), 0);
  NCHECK(IS_INF(-PI/0.0), 0);
  CHECK(IS_INF(PI), 0);

  CHECK(IS_POW2(0), 0);
  CHECK(IS_POW2(1), 1);
  CHECK(IS_POW2(2), 1);
  CHECK(IS_POW2(3), 0);
  CHECK(IS_POW2(31), 0);
  CHECK(IS_POW2(64), 1);
  CHECK(IS_POW2(1 << 16), 1);

  CHECK(log2i(3), 1);
  CHECK(log2i(4), 2);
  CHECK(log2i(5), 2);
  CHECK(log2i(7), 2);
  CHECK(log2i(8), 3);
  CHECK(log2i(12), 3);
  CHECK(log2i(511), 8);
  CHECK(log2i(512), 9);

  CHECK(round_up_pow2(0), 1);
  CHECK(round_up_pow2(3), 4);
  CHECK(round_up_pow2(4), 4);
  CHECK(round_up_pow2(100), 128);

  CHECK(absf(-1.f), 1.f);
  CHECK(absf(10.f), 10.f);
  CHECK(absf(-3.14159f), (float)3.14159f);

  CHECK(eq_eps(MDEG2RAD(90.0), PI*0.5, 1.e-8), 1);
  CHECK(eq_eps(MDEG2RAD(45.0), PI*0.25, 1.e-8), 1);
  CHECK(eq_eps(MRAD2DEG(PI), 180.0, 1.e-8), 1);
  CHECK(eq_eps(MRAD2DEG(PI*0.75), 135.0, 1.e-8), 1);
  CHECK(eq_eps(MRAD2DEG(MDEG2RAD(70.0)), 70.0, 1.e-8), 1);

  FOR_EACH(i, 0, 16) {
    double d, c, s, tmp;
    d = ((float)rand() / (float)RAND_MAX) * PI;
    c = cos(d);
    s = sin(d);
    tmp = cos2sin(c);
    CHECK(eq_eps(s, tmp, 1.0e-8) || eq_eps(s, -tmp, 1.0e-8), 1);
    tmp = sin2cos(s);
    CHECK(eq_eps(c, tmp, 1.0e-8) || eq_eps(c, -tmp, 1.0e-8), 1);
  }

  CHECK(signf(-3.14159f), -1.f);
  CHECK(signf(1.23f), 1.f);
  CHECK(signf(0.f), 1.f);
  return 0;
}

